import copy


def tag_untaged_images(definition, tag):
    new_definition = copy.deepcopy(definition)
    for container in iterate_container_definitions(new_definition):
        container['image'] = tag_untaged_image(container['image'], tag)
    return new_definition


def iterate_container_definitions(new_definition):
    return get_crawler(new_definition).get_container_definitions()


def add_host_volumes(definition, host_volumes):
    new_definition = copy.deepcopy(definition)
    for name, path in host_volumes.items():
        volumes = get_crawler(new_definition).get_volume_definitions()
        volumes.append({'name': name, 'hostPath': {'path': str(path)}})
    return new_definition


def get_crawler(definition):
    return CRAWLER_CLASS_MAP[definition['kind']](definition)


class BaseCrawler:
    def __init__(self, definition):
        self.definition = definition

    def get_container_definitions(self):
        return self.get_pod_spec()['containers']

    def get_volume_definitions(self):
        return self.get_pod_spec().setdefault('volumes', [])

    def get_pod_spec(self):
        raise NotImplementedError


class DefinitionWithPodTemplateCrawler(BaseCrawler):
    def get_pod_spec(self):
        return self.get_pod_crawler().get_pod_spec()

    def get_pod_crawler(self):
        return PodCrawler(self.definition['spec']['template'])


class PodCrawler(BaseCrawler):


    def get_pod_spec(self):
        return self.definition['spec']


def tag_untaged_image(image, tag):
    if ':' in image:
        return image
    else:
        return image + ':' + tag


CRAWLER_CLASS_MAP = {
    'Job': DefinitionWithPodTemplateCrawler,
    'Deployment': DefinitionWithPodTemplateCrawler,
    'Pod': PodCrawler,
}



#!/usr/bin/env python
import optparse
import pathlib

from kubepy import appliers
from kubepy import base_commands


class InstallAllCommand(base_commands.BaseCommand):
    def handle(self, args, options):
        directory_strings = options.directories or ['.']
        directories = [pathlib.Path(directory_string).resolve() for directory_string in directory_strings]
        applier = appliers.DirectoriesApplier(directories, options)
        applier.apply_all()

    def get_optparser(self):
        parser = optparse.OptionParser(
            usage="usage: %prog [options] [name ...]",
            epilog="Apply definitions from given directory."
        )
        parser.add_option(
            '--directory', dest='directories', action='append',
            help='installs definitions from this directory, can be defined multiple times to override definitions.')
        base_commands.add_container_options(parser)
        return parser


def run():
    InstallAllCommand().run()


if __name__ == '__main__':
    run()



import collections
import contextlib
import pathlib

import itertools
import yaml

from kubepy import definition_merger


class BaseDefinitionManager(collections.Mapping):
    def __len__(self):
        return len(list(self))

    def __or__(self, other):
        return OverridenDefinitionManager(self, other)


class OverridenDefinitionManager(BaseDefinitionManager):
    def __init__(self, *managers):
        self.managers = managers

    def __getitem__(self, name):
        definitions = list(self.get_inner_definitions(name))
        if not definitions:
            raise KeyError
        else:
            return definition_merger.merge_definitions(*definitions)

    def get_inner_definitions(self, name):
        for manager in self.managers:
            with contextlib.suppress(KeyError):
                yield manager[name]

    def __iter__(self):
        yield from sorted(set(itertools.chain(*(manager.keys() for manager in self.managers))))


class DefinitionManager(BaseDefinitionManager):
    def __init__(self, directory: pathlib.Path):
        self.directory = directory

    def __getitem__(self, name):
        path = self.get_path_from_name(name)
        if path.exists():
            return self.get_yaml_from_file(path)
        else:
            raise KeyError('{} does not exist'.format(path))

    def __iter__(self):
        for path in sorted(self.directory.glob('*.yml')):
            yield path.name[:-4]

    def __len__(self):
        return len(list(self))

    def get_path_from_name(self, name):
        return self.directory / (name + '.yml')

    def get_yaml_from_file(self, path):
        with path.open() as yaml_file:
            return yaml.load(yaml_file)



from pip.req import parse_requirements
from setuptools import find_packages
from setuptools import setup


setup(
    name='kubepy',
    version='1.5.0',
    description='Python wrapper on kubectl that makes deploying easy.',
    author='Jakub Skiepko',
    author_email='it@socialwifi.com',
    url='https://github.com/socialwifi/kubepy',
    packages=find_packages(),
    install_requires=[str(ir.req) for ir in parse_requirements('base_requirements.txt', session=False)],
    entry_points={
        'console_scripts': [
            'kubepy-apply-all = kubepy.commands.apply_all:run',
            'kubepy-apply-one = kubepy.commands.apply_one:run',
        ],
    },
    license='BSD',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ]
)



import itertools
import functools


@functools.singledispatch
def merge_definitions(*definitions):
    return definitions[-1]


@merge_definitions.register(dict)
def merge_dicts(*definitions):
    keys = set(itertools.chain(*(definition.keys() for definition in definitions)))
    result = {}
    for key in keys:
        result[key] = merge_definitions(*(definition[key] for definition in definitions if key in definition))
    return result


@merge_definitions.register(list)
@merge_definitions.register(tuple)
def merge_lists(*definitions):
    named_results = {}
    unmerged_results = []
    for definition in itertools.chain(*definitions):
        if hasattr(definition, '__getitem__') and 'name' in definition:
            name = definition['name']
            if name in named_results:
                named_results[name].append(definition)
            else:
                new_result = [definition]
                named_results[name] = new_result
                unmerged_results.append(new_result)
        else:
            unmerged_results.append([definition])
    return list(merge_definitions(*results) for results in unmerged_results)
